import random
import math
import pygame
import logging
from config.colors import COLORS
from datetime import datetime
import json
from enum import Enum

# Configure logger for NPC class
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Create console handler with a higher log level
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# Create formatter
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)

# Add the console handler to the logger
logger.addHandler(ch)


class MemoryType(Enum):
    OBSERVATION = "observation"
    INTERACTION = "interaction"
    WEATHER = "weather"
    TIME = "time"
    SOCIAL = "social"
    ACTIVITY = "activity"


class Memory:
    def __init__(self, event, memory_type, importance=1.0):
        self.event = event
        self.timestamp = datetime.now().isoformat()
        self.type = memory_type
        self.importance = importance  # 0.0 to 1.0
        self.age = 0  # Will increase over time


class NPC:
    # List of first names and surnames for more realistic naming
    FIRST_NAMES = [
        "Emma",
        "Liam",
        "Olivia",
        "Noah",
        "Ava",
        "Oliver",
        "Isabella",
        "William",
        "Sophia",
        "James",
        "Mia",
        "Benjamin",
        "Charlotte",
        "Lucas",
        "Amelia",
        "Mason",
        "Harper",
        "Ethan",
        "Evelyn",
        "Alexander",
        "Abigail",
        "Henry",
        "Emily",
        "Sebastian",
        "Elizabeth",
        "Jack",
        "Sofia",
        "Owen",
        "Avery",
        "Daniel",
        "Ella",
        "Matthew",
        "Scarlett",
        "Joseph",
        "Victoria",
    ]

    SURNAMES = [
        "Smith",
        "Johnson",
        "Williams",
        "Brown",
        "Jones",
        "Garcia",
        "Miller",
        "Davis",
        "Rodriguez",
        "Martinez",
        "Hernandez",
        "Lopez",
        "Gonzalez",
        "Wilson",
        "Anderson",
        "Thomas",
        "Taylor",
        "Moore",
        "Jackson",
        "Martin",
        "Lee",
        "Perez",
        "Thompson",
        "White",
        "Harris",
        "Sanchez",
        "Clark",
        "Ramirez",
        "Lewis",
        "Robinson",
        "Walker",
        "Young",
        "Allen",
        "King",
    ]

    def __init__(self, x, y, name):
        self.x = x
        self.y = y
        self.name = name
        self.radius = 15
        self.speed = 1.0
        self.interaction_radius = 50
        self.memories = []
        self.memory_limit = 50
        self.last_weather_check = pygame.time.get_ticks()
        self.weather_check_interval = 300
        self.long_term_memories = {
            "player_name": None,
            "conversations": [],
            "important_facts": {},
            "relationships": {},
            "daily_routine": [],
            "preferences": self.generate_preferences(),
        }
        self.home_x = x
        self.home_y = y
        self.mood = random.choice(["happy", "neutral", "thoughtful", "busy"])
        self.current_activity = None
        self.color = self.generate_npc_color()
        # Simple movement variables
        self.move_angle = random.uniform(0, 2 * math.pi)
        self.move_timer = 0
        self.is_talking = False  # Add this flag
        self.original_pos = None  # Store position when conversation starts

    def generate_preferences(self):
        """Generate random preferences for the NPC"""
        return {
            "favorite_time": random.choice(
                ["morning", "afternoon", "evening", "night"]
            ),
            "favorite_weather": random.choice(["sunny", "rainy", "cloudy"]),
            "favorite_activity": random.choice(
                ["walking", "talking", "observing", "working"]
            ),
            "personality": random.choice(["outgoing", "shy", "curious", "reserved"]),
        }

    def generate_npc_color(self):
        """Generate a pleasant, unique color for the NPC"""
        # Base colors for villagers (warm, earthy tones)
        base_colors = [
            (139, 69, 19),  # Saddle brown
            (160, 82, 45),  # Sienna
            (205, 133, 63),  # Peru
            (210, 105, 30),  # Chocolate
            (184, 134, 11),  # Dark goldenrod
            (165, 42, 42),  # Brown
            (128, 70, 27),  # Russet
            (139, 90, 43),  # Leather
            (153, 101, 21),  # Golden brown
            (130, 102, 68),  # Beaver
        ]
        return random.choice(base_colors)

    def update(self, world, npcs, player, current_time=None, weather=None):
        """Movement system with conversation handling"""
        # If in conversation, stay still
        if self.is_talking:
            logger.debug(f"NPC {self.name} is in conversation, staying still")
            return

        # Debug current position and planned movement
        #logger.debug(
        #    f"NPC {self.name} current position: ({int(self.x)}, {int(self.y)})"
        #)

        # Check if on or near a road/path
        on_path = False
        nearest_path = None
        nearest_path_dist = float("inf")

        for obj in world.objects:
            if getattr(obj, "type", None) in ["road", "path"]:
                dx = self.x - obj.x
                dy = self.y - obj.y
                dist = math.sqrt(dx * dx + dy * dy)
                if dist < (self.radius + obj.radius):
                    on_path = True
                    self.speed = 1.5  # Slightly faster on paths
                elif dist < nearest_path_dist:
                    nearest_path_dist = dist
                    nearest_path = obj

        if not on_path:
            self.speed = 1.0  # Normal speed off paths
            # Try to move towards nearest path occasionally
            if (
                nearest_path and random.random() < 0.02
            ):  # 2% chance to head towards path
                angle = math.atan2(nearest_path.y - self.y, nearest_path.x - self.x)
                self.move_angle = angle
                #logger.debug(f"NPC {self.name} heading towards path")

        # Check if NPC is stuck inside a house
        for obj in world.objects:
            # Skip roads for collision
            if getattr(obj, "type", None) in ["road", "path"]:
                continue

            dx = self.x - obj.x
            dy = self.y - obj.y
            distance = math.sqrt(dx * dx + dy * dy)
            if distance < (self.radius + obj.radius):
                logger.debug(
                    f"NPC {self.name} is inside object {obj.__class__.__name__} at ({int(obj.x)}, {int(obj.y)})"
                )
                # Try to escape by moving away from object center
                escape_angle = math.atan2(dy, dx)
                self.x = obj.x + math.cos(escape_angle) * (obj.radius + self.radius + 5)
                self.y = obj.y + math.sin(escape_angle) * (obj.radius + self.radius + 5)
                logger.debug(
                    f"NPC {self.name} escaped to ({int(self.x)}, {int(self.y)})"
                )
                return

        # Try to move in X direction first
        move_x = math.cos(self.move_angle) * self.speed
        new_x = self.x + move_x
        new_x = max(self.radius, min(world.size - self.radius, new_x))

        # Check X movement with more lenient collision
        can_move_x = True
        for obj in world.objects:
            # Skip roads for collision
            if getattr(obj, "type", None) in ["road", "path"]:
                continue

            dx = new_x - obj.x
            dy = self.y - obj.y
            distance = math.sqrt(dx * dx + dy * dy)
            if distance < (self.radius + obj.radius + 2):  # Small buffer
                can_move_x = False
                logger.debug(
                    f"X collision with {obj.__class__.__name__} at ({int(obj.x)}, {int(obj.y)})"
                )
                break

        if can_move_x:
            self.x = new_x
            #logger.debug(f"NPC {self.name} moved X to {int(self.x)}")

        # Then try Y movement separately
        move_y = math.sin(self.move_angle) * self.speed
        new_y = self.y + move_y
        new_y = max(self.radius, min(world.size - self.radius, new_y))

        # Check Y movement with more lenient collision
        can_move_y = True
        for obj in world.objects:
            # Skip roads for collision
            if getattr(obj, "type", None) in ["road", "path"]:
                continue

            dx = self.x - obj.x
            dy = new_y - obj.y
            distance = math.sqrt(dx * dx + dy * dy)
            if distance < (self.radius + obj.radius + 2):  # Small buffer
                can_move_y = False
                logger.debug(
                    f"Y collision with {obj.__class__.__name__} at ({int(obj.x)}, {int(obj.y)})"
                )
                break

        if can_move_y:
            self.y = new_y
            logger.debug(f"NPC {self.name} moved Y to {int(self.y)}")

        # If both movements failed, try to move away from nearby objects
        if not (can_move_x or can_move_y):
            # Find nearest object
            nearest_obj = None
            nearest_dist = float("inf")
            for obj in world.objects:
                dx = self.x - obj.x
                dy = self.y - obj.y
                dist = math.sqrt(dx * dx + dy * dy)
                if dist < nearest_dist:
                    nearest_dist = dist
                    nearest_obj = obj

            if nearest_obj:
                # Move away from nearest object
                dx = self.x - nearest_obj.x
                dy = self.y - nearest_obj.y
                angle = math.atan2(dy, dx)
                self.move_angle = angle
                logger.debug(
                    f"NPC {self.name} moving away from object at angle {angle}"
                )
            else:
                # Random new direction if no nearby objects
                self.move_angle = random.uniform(0, 2 * math.pi)
                logger.debug(
                    f"NPC {self.name} changing to random direction {self.move_angle}"
                )

        # Update memories and observations
        self.age_memories()
        self.observe_environment(world, npcs, player, current_time, weather)
        self.update_activity(current_time)

    def age_memories(self):
        """Age memories and remove old ones"""
        for memory in self.memories:
            memory.age += 1

        # Remove very old memories unless they're important
        self.memories = [m for m in self.memories if m.age < 1000 or m.importance > 0.8]

    def observe_environment(self, world, npcs, player, current_time, weather):
        """Generate diverse observations about the environment"""
        # Time-based observations
        if current_time:
            time_of_day = self.get_time_of_day(current_time)
            if random.random() < 0.1:
                self.add_memory(
                    f"It's {time_of_day} now", MemoryType.TIME, importance=0.3
                )

        # Weather observations
        if (
            weather
            and (self.last_weather_check + self.weather_check_interval)
            <= pygame.time.get_ticks()
        ):
            self.add_memory(
                f"The weather is {weather}", MemoryType.WEATHER, importance=0.4
            )
            self.last_weather_check = pygame.time.get_ticks()

        # Social observations
        for npc in npcs:
            if npc != self and self.distance_to(npc) < 100:
                if random.random() < 0.05:
                    activity = random.choice(
                        [
                            "walking around",
                            "looking at the surroundings",
                            "heading somewhere",
                            "working",
                        ]
                    )
                    self.add_memory(
                        f"Saw {npc.name} {activity}", MemoryType.SOCIAL, importance=0.6
                    )

        # Environment observations
        nearby_objects = self.get_nearby_objects(world)
        if nearby_objects and random.random() < 0.1:
            obj = random.choice(nearby_objects)
            observation = self.generate_object_observation(obj)
            self.add_memory(observation, MemoryType.OBSERVATION, importance=0.5)

    def generate_object_observation(self, obj):
        """Generate varied observations about objects"""
        if obj.type == "tree":
            return random.choice(
                [
                    "The trees are providing nice shade",
                    "The leaves are rustling in the wind",
                    "This tree looks particularly old",
                ]
            )
        elif obj.type == "flower":
            return random.choice(
                [
                    "The flowers are blooming beautifully",
                    "There's a pleasant floral scent in the air",
                    "These flowers add color to the village",
                ]
            )
        elif obj.type == "house":
            return random.choice(
                [
                    "That's a well-maintained house",
                    "Someone's home looks cozy",
                    "The house has an interesting design",
                ]
            )
        elif obj.type == "path":
            return random.choice(
                [
                    "The path is well-worn from use",
                    "This path connects different parts of the village",
                    "People often use this route",
                ]
            )
        return f"I noticed a {obj.type} nearby"

    def add_memory(self, event, memory_type, importance=1.0):
        """Add a new memory with type and importance"""
        memory = Memory(event, memory_type, importance)
        self.memories.append(memory)

        # Keep memories within limit, remove oldest and least important first
        if len(self.memories) > self.memory_limit:
            self.memories.sort(key=lambda m: m.importance * (1.0 / (1.0 + m.age)))
            self.memories.pop(0)

    def get_memory_context(self):
        """Get enhanced context from memories for conversation"""
        context = []

        # Add personality and preferences (safely)
        personality = self.long_term_memories.get("preferences", {}).get(
            "personality", "friendly"
        )
        favorite_activity = self.long_term_memories.get("preferences", {}).get(
            "favorite_activity", "chatting"
        )
        context.append(f"You are {personality} and prefer {favorite_activity}")

        # Add current mood and activity
        context.append(
            f"You are currently {self.mood} and {self.current_activity or 'not doing anything specific'}"
        )

        # Add player knowledge
        if self.long_term_memories["player_name"]:
            context.append(
                f"You know the player as {self.long_term_memories['player_name']}"
            )

        # Add recent memories, weighted by type and importance
        weighted_memories = sorted(
            self.memories,
            key=lambda m: m.importance * (1.0 / (1.0 + m.age)),
            reverse=True,
        )[
            :5
        ]  # Take top 5 memories

        for memory in weighted_memories:
            context.append(memory.event)

        return " ".join(context)

    def get_nearby_objects(self, world):
        """Get objects within observation range"""
        nearby = []
        for obj in world.objects:
            if self.distance_to_pos(obj.x, obj.y) < 150:  # Observation range
                nearby.append(obj)
        return nearby

    def distance_to(self, other):
        """Calculate distance to another entity"""
        return ((self.x - other.x) ** 2 + (self.y - other.y) ** 2) ** 0.5

    def distance_to_pos(self, x, y):
        """Calculate distance to a position"""
        dx = x - self.x
        dy = y - self.y
        return math.sqrt(dx * dx + dy * dy)

    def get_time_of_day(self, current_time):
        """Convert time to period of day"""
        hour = current_time.hour
        if 5 <= hour < 12:
            return "morning"
        elif 12 <= hour < 17:
            return "afternoon"
        elif 17 <= hour < 21:
            return "evening"
        else:
            return "night"

    def update_activity(self, current_time):
        """Update NPC's current activity based on time"""
        if not current_time:
            return

        hour = current_time.hour
        if 6 <= hour < 10:
            self.current_activity = "starting the day"
        elif 10 <= hour < 12:
            self.current_activity = "working in the morning"
        elif 12 <= hour < 14:
            self.current_activity = "having lunch"
        elif 14 <= hour < 18:
            self.current_activity = "working in the afternoon"
        elif 18 <= hour < 21:
            self.current_activity = "spending evening time"
        else:
            self.current_activity = "resting"

    def remember_conversation(self, player_message, npc_response):
        """Store important conversation details"""
        timestamp = datetime.now().isoformat()
        self.long_term_memories["conversations"].append(
            {
                "timestamp": timestamp,
                "player_said": player_message,
                "npc_said": npc_response,
            }
        )

        # Try to extract player name if mentioned
        if "my name is" in player_message.lower():
            try:
                name = player_message.lower().split("my name is")[1].strip()
                self.long_term_memories["player_name"] = name
                self.long_term_memories["important_facts"]["player_name"] = name
            except:
                pass

    def choose_next_action(self, world):
        """Choose what to do next based on time and location"""
        logger.debug(f"NPC {self.name} choosing next action")

        time_of_day = self.get_time_of_day(datetime.now())
        current_weather = getattr(self, "current_weather", "sunny")

        action = random.choice(
            ["wander", "visit_point_of_interest", "return_home", "wait"]
        )

        logger.debug(f"NPC {self.name} chose action: {action}")

        if action == "wander":
            angle = random.uniform(0, 2 * math.pi)
            distance = random.uniform(50, self.wander_radius)
            target_x = self.x + math.cos(angle) * distance
            target_y = self.y + math.sin(angle) * distance
            logger.debug(
                f"NPC {self.name} wandering to ({int(target_x)}, {int(target_y)})"
            )
            self.set_target(target_x, target_y)

        elif action == "visit_point_of_interest":
            # Find nearby interesting points (buildings, wells, etc.)
            points = self.find_points_of_interest(world)
            if points:
                point = random.choice(points)
                self.set_target(point.x, point.y)
            else:
                self.state = "waiting"
                self.wait_time = random.randint(60, 180)

        elif action == "return_home":
            self.set_target(self.home_x, self.home_y)

        elif action == "wait":
            self.state = "waiting"
            self.wait_time = random.randint(60, 180)

    def set_target(self, x, y):
        """Set a new movement target"""
        self.target_x = max(0, min(x, 3000))  # Assume world size is 3000
        self.target_y = max(0, min(y, 3000))
        self.state = "walking"
        self.state_timer = 0

    def find_points_of_interest(self, world):
        """Find interesting points within observation range"""
        points = []
        for obj in world.objects:
            if (
                obj.type in ["house", "well", "tree"]
                and self.distance_to_pos(obj.x, obj.y) < self.wander_radius
            ):
                points.append(obj)
        return points

    def force_random_movement(self, world):
        """Force NPC to move in a random direction"""
        angle = random.uniform(0, 2 * math.pi)
        distance = 50  # Short distance for default movement

        # Try 8 different directions if initial direction is blocked
        for _ in range(8):
            target_x = self.x + math.cos(angle) * distance
            target_y = self.y + math.sin(angle) * distance

            # Check if movement is possible
            if not world.check_collision(target_x, target_y, self.radius):
                self.target_x = target_x
                self.target_y = target_y
                self.state = "walking"
                self.state_timer = 0
                logger.debug(
                    f"NPC {self.name} forced movement to ({int(target_x)}, {int(target_y)})"
                )
                return

            # Try next direction
            angle += math.pi / 4

        logger.debug(f"NPC {self.name} couldn't find valid movement direction")

    def move_to_target(self, world):
        """Handle movement towards target"""
        dx = self.target_x - self.x
        dy = self.target_y - self.y
        distance = math.sqrt(dx * dx + dy * dy)

        logger.debug(f"NPC {self.name} walking to target. Distance: {distance:.2f}")

        if distance < self.speed:  # Reached target
            self.x = self.target_x
            self.y = self.target_y
            self.state = "idle"
            self.state_timer = 0
            self.target_x = None
            self.target_y = None
            logger.debug(f"NPC {self.name} reached target")
        else:
            # Smooth movement towards target
            move_x = (dx / distance) * self.speed
            move_y = (dy / distance) * self.speed

            # Check for collision before moving
            new_x = self.x + move_x
            new_y = self.y + move_y

            if not world.check_collision(new_x, new_y, self.radius):
                self.x = new_x
                self.y = new_y
                logger.debug(f"NPC {self.name} moved to ({int(self.x)}, {int(self.y)})")
            else:
                logger.debug(f"NPC {self.name} collision detected")
                self.force_random_movement(world)  # Try moving in a different direction

    def debug_collision(self, world, x, y):
        """Debug helper to check what's causing collisions"""
        # Check world bounds
        if (
            x < self.radius
            or x > world.size - self.radius
            or y < self.radius
            or y > world.size - self.radius
        ):
            logger.debug(f"World bounds collision at ({int(x)}, {int(y)})")
            return True

        # Check objects
        for obj in world.objects:
            dx = x - obj.x
            dy = y - obj.y
            distance = math.sqrt(dx * dx + dy * dy)
            if distance < (self.radius + obj.radius):
                logger.debug(f"Object collision with {obj} at ({int(x)}, {int(y)})")
                return True

        return False
