import pygame
import asyncio
import math
import random
from ollama import Client
import logging
from datetime import datetime

from config.settings import WINDOW_SIZE, OLLAMA_URL, OLLAMA_MODEL
from config.colors import COLORS
from entities.npc import NPC
from utils.memory import Memory
from world.world import World
from world.camera import Camera
from entities.player import Player
from utils.dialog import DialogBox
from utils.save_manager import SaveManager
from menu.main_menu import MainMenu
from ui.header_bar import HeaderBar
from utils.name_generator import NameGenerator
from ui.footer_bar import FooterBar
from effects.weather import WeatherEffect

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize Pygame
pygame.init()
screen = pygame.display.set_mode(WINDOW_SIZE)
pygame.display.set_caption("LMMRPG")


def draw_world_object(screen, obj, x, y):
    """Draw a world object with appropriate shape and color"""
    # Ensure color is a valid RGB tuple
    color = obj.color if isinstance(obj.color, tuple) else (0, 0, 0)

    if obj.type == "path":
        # Draw paths as rectangles
        pygame.draw.rect(screen, color, (x - 10, y - 10, 20, 20))
    elif obj.type == "water":
        # Draw water as blue rectangles with slight transparency
        surface = pygame.Surface((20, 20), pygame.SRCALPHA)
        pygame.draw.rect(surface, (*color, 180), (0, 0, 20, 20))  # Add alpha channel
        screen.blit(surface, (x - 10, y - 10))
    elif obj.type == "house":
        # Draw houses as brown rectangles with a triangular roof
        pygame.draw.rect(
            screen,
            color,
            (x - obj.radius, y - obj.radius, obj.radius * 2, obj.radius * 2),
        )
        # Roof
        pygame.draw.polygon(
            screen,
            (139, 69, 19),  # Darker brown for roof
            [
                (x - obj.radius - 5, y - obj.radius),
                (x + obj.radius + 5, y - obj.radius),
                (x, y - obj.radius - 15),
            ],
        )
    elif obj.type == "well":
        # Draw wells as circles with a darker border
        pygame.draw.circle(screen, color, (int(x), int(y)), obj.radius)
        pygame.draw.circle(screen, (100, 100, 100), (int(x), int(y)), obj.radius, 2)
    elif obj.type == "tree":
        # Draw trees as green circles with brown trunks
        # Trunk
        pygame.draw.rect(screen, (139, 69, 19), (x - 3, y - 5, 6, 10))  # Brown
        # Leaves
        pygame.draw.circle(screen, color, (int(x), int(y - 10)), obj.radius)
    elif obj.type in ["flower", "mushroom", "tall_grass", "berry_bush"]:
        # Draw small decorative elements
        if obj.type == "flower":
            # Flowers as small colored circles with a center
            pygame.draw.circle(screen, color, (int(x), int(y)), obj.radius)
            pygame.draw.circle(
                screen, (255, 255, 0), (int(x), int(y)), 2
            )  # Yellow center
        elif obj.type == "mushroom":
            # Mushrooms as small stems with caps
            pygame.draw.rect(screen, (210, 180, 140), (x - 1, y - 3, 2, 6))  # Stem
            pygame.draw.circle(screen, color, (int(x), int(y - 3)), 4)  # Cap
        elif obj.type == "tall_grass":
            # Tall grass as small vertical lines
            for i in range(-2, 3):
                pygame.draw.line(
                    screen, color, (x + i, y), (x + i, y - random.randint(5, 8))
                )
        elif obj.type == "berry_bush":
            # Berry bushes as green circles with red dots
            pygame.draw.circle(
                screen, (0, 100, 0), (int(x), int(y)), obj.radius
            )  # Bush
            for _ in range(3):  # Add berries
                berry_x = x + random.randint(-3, 3)
                berry_y = y + random.randint(-3, 3)
                pygame.draw.circle(screen, (139, 0, 0), (int(berry_x), int(berry_y)), 2)
    else:
        # Default drawing for unknown objects
        pygame.draw.circle(screen, color, (int(x), int(y)), obj.radius)


def draw_header(screen, world):
    font = pygame.font.Font(None, 32)
    header_height = 40

    # Draw header background
    pygame.draw.rect(screen, (0, 0, 0, 180), (0, 0, WINDOW_SIZE[0], header_height))

    # Draw seed information
    seed_text = f"World Seed: {world.seed}"
    seed_surface = font.render(seed_text, True, (255, 255, 255))
    screen.blit(seed_surface, (10, 10))


def draw_chat_input(screen, input_text, active):
    # Draw chat input box at the very bottom
    input_box_height = 40
    input_box_width = WINDOW_SIZE[0] * 0.8
    x = (WINDOW_SIZE[0] - input_box_width) / 2
    y = WINDOW_SIZE[1] - input_box_height - 10  # Always at bottom

    # Draw input box background
    pygame.draw.rect(
        screen,
        (0, 0, 0, 180) if active else (50, 50, 50, 180),
        (x, y, input_box_width, input_box_height),
    )

    # Draw input text
    font = pygame.font.Font(None, 32)
    if input_text:
        text_surface = font.render(input_text, True, (255, 255, 255))
    else:
        text_surface = font.render(
            "Type your message and press Enter...", True, (150, 150, 150)
        )

    screen.blit(text_surface, (x + 10, y + 10))


async def main():
    pygame.init()
    screen = pygame.display.set_mode(WINDOW_SIZE)
    pygame.display.set_caption("LMMRPG")

    save_manager = SaveManager()
    menu = MainMenu(save_manager)

    # Initialize save counter
    save_counter = 0
    save_cooldown = 300

    # Start with menu
    current_state = "menu"
    world = None
    player = None
    camera = None

    # Add time and weather system
    current_time = datetime.now()
    weather_conditions = ["sunny", "cloudy", "rainy"]
    current_weather = random.choice(weather_conditions)
    weather_change_counter = 0

    header_bar = HeaderBar()
    footer_bar = FooterBar()
    weather_effect = WeatherEffect(WINDOW_SIZE[0], WINDOW_SIZE[1])

    # Update world generation to use proper names
    def generate_npcs(world):
        for village_x, village_y in world.village_centers:
            num_npcs = random.randint(3, 6)
            for _ in range(num_npcs):
                x = village_x + random.randint(-100, 100)
                y = village_y + random.randint(-100, 100)
                npc = NPC(x, y, NameGenerator.generate_name())
                world.npcs.append(npc)

    running = True
    while running:
        if current_state == "menu":
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
                    continue

                result = menu.handle_input(event)
                if result:
                    action, seed = result
                    if action == "quit":
                        running = False
                    elif action == "new":
                        world = World(seed=seed)
                        # Find valid spawn for player
                        spawn_x, spawn_y = world.find_valid_spawn(
                            world.size // 2, world.size // 2
                        )
                        player = Player(spawn_x, spawn_y)
                        camera = Camera()
                        current_state = "game"
                    elif action == "load":
                        save_data = save_manager.load_world(seed)
                        if save_data:
                            world_data, npc_data = save_data
                            world = World(seed=seed)
                            # Restore NPCs and their memories
                            for npc_info in npc_data:
                                for npc in world.npcs:
                                    if npc.name == npc_info["name"]:
                                        npc.memories = [
                                            Memory(
                                                m["event"],
                                                m["timestamp"],
                                                m["location"],
                                            )
                                            for m in npc_info["memories"]
                                        ]
                                        npc.long_term_memories = npc_info.get(
                                            "long_term_memories", []
                                        )
                            player = Player(world.size // 2, world.size // 2)
                            camera = Camera()
                            current_state = "game"

            menu.draw(screen)
            pygame.display.flip()
            continue

        # Regular game loop
        clock = pygame.time.Clock()
        world = World()
        player = Player(world.size // 2, world.size // 2)
        camera = Camera()
        ollama_client = Client(host=OLLAMA_URL)
        dialog_box = DialogBox()

        # Keep track of the last NPC we talked to
        current_npc = None
        conversation_history = []

        input_text = ""
        chat_active = False

        running = True
        while running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
                elif event.type == pygame.KEYDOWN:
                    if chat_active:
                        if event.key == pygame.K_RETURN and input_text.strip():
                            # Handle chat input
                            dialog_box.show_message("You", input_text)

                            # Add to conversation history
                            conversation_history.append(
                                {"role": "user", "content": input_text}
                            )

                            # Generate NPC response
                            try:
                                # Get memory context from NPC
                                memory_context = current_npc.get_memory_context()
                                context = (
                                    f"You are {current_npc.name}, a villager. "
                                    f"{memory_context} "
                                    f"Recent events: {[m.event for m in current_npc.memories[-3:]]}"
                                )

                                messages = [{"role": "system", "content": context}]
                                messages.extend(conversation_history[-6:])

                                response = ollama_client.chat(
                                    model=OLLAMA_MODEL, messages=messages
                                )

                                # Store the conversation in NPC's memory
                                current_npc.remember_conversation(
                                    input_text, response["message"]["content"]
                                )

                                # Add NPC response to history
                                conversation_history.append(
                                    {
                                        "role": "assistant",
                                        "content": response["message"]["content"],
                                    }
                                )

                                # Show NPC response
                                dialog_box.show_message(
                                    current_npc.name, response["message"]["content"]
                                )

                            except Exception as e:
                                logger.error(f"Ollama error details: {str(e)}")
                                if hasattr(e, "response"):
                                    logger.error(
                                        f"Response status: {e.response.status_code}"
                                    )
                                    logger.error(f"Response content: {e.response.text}")
                                dialog_box.show_message(
                                    current_npc.name,
                                    f"I'm sorry, I didn't quite catch that... (Error: {str(e)})",
                                )

                            input_text = ""  # Clear input

                        elif event.key == pygame.K_BACKSPACE:
                            input_text = input_text[:-1]
                        elif event.key == pygame.K_ESCAPE:
                            chat_active = False
                            dialog_box.hide()
                            current_npc = None
                            conversation_history = []
                        elif event.key == pygame.K_UP:
                            dialog_box.scroll_up()
                        elif event.key == pygame.K_DOWN:
                            dialog_box.scroll_down()
                        elif len(input_text) < 50 and event.unicode.isprintable():
                            input_text += event.unicode
                    else:
                        # Only handle non-chat keys when not chatting
                        if event.key == pygame.K_SPACE:
                            # Find nearby NPCs
                            nearby_npcs = []
                            for npc in world.npcs:
                                dist = math.sqrt(
                                    (npc.x - player.x) ** 2 + (npc.y - player.y) ** 2
                                )
                                if dist <= npc.interaction_radius:
                                    nearby_npcs.append(npc)

                            if nearby_npcs:
                                nearest_npc = min(
                                    nearby_npcs,
                                    key=lambda n: (
                                        (n.x - player.x) ** 2 + (n.y - player.y) ** 2
                                    ),
                                )

                                if not chat_active:
                                    # Start new conversation
                                    current_npc = nearest_npc
                                    chat_active = True
                                    conversation_history = []

                                    # Initial greeting
                                    try:
                                        context = f"You are {current_npc.name}, a villager. Your recent memories: {[m.event for m in current_npc.memories[-3:]]}"
                                        response = ollama_client.chat(
                                            model=OLLAMA_MODEL,
                                            messages=[
                                                {"role": "system", "content": context},
                                                {"role": "user", "content": "Hello!"},
                                            ],
                                        )
                                        dialog_box.show_message(
                                            current_npc.name,
                                            response["message"]["content"],
                                        )
                                        conversation_history.append(
                                            {
                                                "role": "assistant",
                                                "content": response["message"][
                                                    "content"
                                                ],
                                            }
                                        )
                                    except Exception as e:
                                        logger.error(f"Ollama error: {e}")
                                        dialog_box.show_message(
                                            current_npc.name,
                                            "Hello traveler! (Ollama connection failed)",
                                        )
                            else:
                                dialog_box.show_message(
                                    "Info", "No NPCs nearby to talk to!"
                                )

            # Only update player and world when not chatting
            if not chat_active:
                keys = pygame.key.get_pressed()
                player.update(keys, world)
                camera.update(player)

                for npc in world.npcs:
                    npc.update(world, world.npcs, player)

            # Drawing
            screen.fill((34, 139, 34))  # Background grass

            # Draw world objects by layer
            for obj in world.objects:
                screen_x = obj.x - camera.x
                screen_y = obj.y - camera.y
                if (
                    -50 <= screen_x <= WINDOW_SIZE[0] + 50
                    and -50 <= screen_y <= WINDOW_SIZE[1] + 50
                ):
                    draw_world_object(screen, obj, screen_x, screen_y)

            # Draw NPCs
            for npc in world.npcs:
                screen_x = npc.x - camera.x
                screen_y = npc.y - camera.y
                if 0 <= screen_x <= WINDOW_SIZE[0] and 0 <= screen_y <= WINDOW_SIZE[1]:
                    # Draw interaction radius for nearby NPCs
                    dist_to_player = math.sqrt(
                        (npc.x - player.x) ** 2 + (npc.y - player.y) ** 2
                    )
                    if dist_to_player <= npc.interaction_radius:
                        pygame.draw.circle(
                            screen,
                            (255, 255, 255, 50),
                            (int(screen_x), int(screen_y)),
                            npc.interaction_radius,
                            1,
                        )

                    # Draw NPC
                    pygame.draw.circle(
                        screen, npc.color, (int(screen_x), int(screen_y)), npc.radius
                    )
                    pygame.draw.circle(
                        screen,
                        npc.color,
                        (int(screen_x), int(screen_y - npc.radius - 5)),
                        npc.radius - 5,
                    )

            # Draw player
            player_screen_x = WINDOW_SIZE[0] // 2
            player_screen_y = WINDOW_SIZE[1] // 2
            pygame.draw.circle(
                screen, player.color, (player_screen_x, player_screen_y), player.radius
            )
            pygame.draw.circle(
                screen,
                player.color,
                (player_screen_x, player_screen_y - player.radius - 5),
                player.radius - 5,
            )

            # Draw weather effects last (on top of everything except UI)
            weather_effect.draw(screen, current_weather)

            # Draw UI elements after weather
            header_bar.draw(
                screen, current_time, current_weather, (player.x, player.y), world.size
            )
            footer_bar.draw(screen, world)

            # Draw dialog box
            dialog_box.draw(screen)

            # Draw chat input if active
            if chat_active:
                draw_chat_input(screen, input_text, True)

            pygame.display.flip()
            clock.tick(60)

            # Handle manual save on key press
            keys = pygame.key.get_pressed()
            if keys[pygame.K_F5]:  # F5 to save
                save_manager.save_world(world)
                dialog_box.show_message("System", "Game saved!")

            # Auto-save less frequently
            save_counter += 1
            if save_counter >= save_cooldown:
                save_manager.save_world(world)
                save_counter = 0

            # Update NPC memories immediately after conversation
            if chat_active and current_npc:
                current_npc.remember_conversation(input_text, dialog_box.message)
                save_manager.save_world(world)  # Save immediately after memory update

            # Update time every minute
            if datetime.now().minute != current_time.minute:
                current_time = datetime.now()

            # Change weather occasionally
            weather_change_counter += 1
            if weather_change_counter >= 3600:  # Change weather every ~1 minute
                current_weather = random.choice(weather_conditions)
                weather_change_counter = 0

            # Update NPCs with time and weather
            for npc in world.npcs:
                npc.current_weather = current_weather  # Set weather before update
                npc.update(world, world.npcs, player, current_time, current_weather)

            # Update world's weather
            world.current_weather = current_weather

    # When quitting
    pygame.quit()


if __name__ == "__main__":
    asyncio.run(main())
