import pygame
import random
from config.settings import WINDOW_SIZE
from config.colors import COLORS


class MainMenu:
    def __init__(self, save_manager):
        self.save_manager = save_manager
        self.font = pygame.font.Font(None, 48)
        self.small_font = pygame.font.Font(None, 32)
        self.selected = 0
        self.input_seed = ""
        self.state = "main"  # main, load, enter_seed
        self.saves = []
        self.bg_color = (0, 0, 0)
        self.text_color = (255, 255, 255)
        self.highlight_color = (255, 255, 0)

    def update_saves(self):
        self.saves = self.save_manager.get_available_saves()

    def draw(self, screen):
        screen.fill(self.bg_color)

        if self.state == "main":
            options = [
                "New Game (Random Seed)",
                "New Game (Enter Seed)",
                "Load Game",
                "Quit",
            ]

            for i, option in enumerate(options):
                color = self.highlight_color if i == self.selected else self.text_color
                text = self.font.render(option, True, color)
                x = WINDOW_SIZE[0] // 2 - text.get_width() // 2
                y = WINDOW_SIZE[1] // 2 - len(options) * 30 + i * 60
                screen.blit(text, (x, y))

        elif self.state == "load":
            # Draw back option
            back_text = self.font.render(
                "Back",
                True,
                self.highlight_color if self.selected == -1 else self.text_color,
            )
            screen.blit(back_text, (20, 20))

            if not self.saves:
                text = self.font.render("No saves found", True, self.text_color)
                screen.blit(
                    text,
                    (WINDOW_SIZE[0] // 2 - text.get_width() // 2, WINDOW_SIZE[1] // 2),
                )
            else:
                for i, save in enumerate(self.saves):
                    color = (
                        self.highlight_color if i == self.selected else self.text_color
                    )
                    text = self.font.render(
                        f"World {save['seed']} - {save['timestamp'][:10]}", True, color
                    )
                    x = WINDOW_SIZE[0] // 2 - text.get_width() // 2
                    y = 100 + i * 60
                    screen.blit(text, (x, y))

        elif self.state == "enter_seed":
            prompt = self.font.render(
                "Enter Seed (numbers only):", True, self.text_color
            )
            seed_text = self.font.render(
                self.input_seed or "...", True, self.highlight_color
            )

            screen.blit(
                prompt,
                (
                    WINDOW_SIZE[0] // 2 - prompt.get_width() // 2,
                    WINDOW_SIZE[1] // 2 - 60,
                ),
            )
            screen.blit(
                seed_text,
                (WINDOW_SIZE[0] // 2 - seed_text.get_width() // 2, WINDOW_SIZE[1] // 2),
            )

            instruction = self.small_font.render(
                "Press Enter to confirm, Escape to cancel", True, self.text_color
            )
            screen.blit(
                instruction,
                (
                    WINDOW_SIZE[0] // 2 - instruction.get_width() // 2,
                    WINDOW_SIZE[1] // 2 + 60,
                ),
            )

    def handle_input(self, event):
        if self.state == "main":
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_UP:
                    self.selected = (self.selected - 1) % 4
                elif event.key == pygame.K_DOWN:
                    self.selected = (self.selected + 1) % 4
                elif event.key == pygame.K_RETURN:
                    if self.selected == 0:  # New Game (Random)
                        return ("new", random.randint(1, 999999))
                    elif self.selected == 1:  # New Game (Seed)
                        self.state = "enter_seed"
                        self.input_seed = ""
                    elif self.selected == 2:  # Load Game
                        self.state = "load"
                        self.selected = 0  # Start with first save selected
                        self.update_saves()
                    elif self.selected == 3:  # Quit
                        return ("quit", None)

        elif self.state == "load":
            if event.type == pygame.KEYDOWN:
                max_index = len(self.saves)
                if event.key == pygame.K_UP:
                    self.selected = (self.selected - 1) % (max_index + 1)
                elif event.key == pygame.K_DOWN:
                    self.selected = (self.selected + 1) % (max_index + 1)
                elif event.key == pygame.K_RETURN:
                    if self.selected == max_index:  # Back option
                        self.state = "main"
                        self.selected = 0
                    elif self.saves:  # Load selected save
                        return ("load", self.saves[self.selected]["seed"])
                elif event.key == pygame.K_ESCAPE:
                    self.state = "main"
                    self.selected = 0

        elif self.state == "enter_seed":
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_RETURN and self.input_seed:
                    return ("new", int(self.input_seed))
                elif event.key == pygame.K_ESCAPE:
                    self.state = "main"
                    self.selected = 1
                elif event.key == pygame.K_BACKSPACE:
                    self.input_seed = self.input_seed[:-1]
                elif event.unicode.isnumeric() and len(self.input_seed) < 6:
                    self.input_seed += event.unicode

        return None
