import os
import json
import logging
from datetime import datetime

# Configure logger for save manager
logger = logging.getLogger(__name__)
logger.setLevel(logging.WARNING)  # Only show warnings and errors


class SaveManager:
    def __init__(self):
        self.save_dir = "saves"
        self.ensure_save_directory()

    def ensure_save_directory(self):
        """Ensure save directory exists"""
        if not os.path.exists(self.save_dir):
            os.makedirs(self.save_dir)

    def get_world_path(self, seed):
        """Get path for a specific world save"""
        return os.path.join(self.save_dir, f"world_{seed}")

    def save_world(self, world):
        """Save world state including NPCs and their memories"""
        try:
            world_dir = self.get_world_path(world.seed)
            if not os.path.exists(world_dir):
                os.makedirs(world_dir)

            # Save world data
            world_data = {
                "seed": world.seed,
                "size": world.size,
                "timestamp": datetime.now().isoformat(),
                "village_centers": world.village_centers,
            }

            with open(os.path.join(world_dir, "world.json"), "w") as f:
                json.dump(world_data, f)

            # Save NPCs and their memories
            npc_data = []
            for npc in world.npcs:
                npc_info = {
                    "name": npc.name,
                    "x": npc.x,
                    "y": npc.y,
                    "home_x": npc.home_x,
                    "home_y": npc.home_y,
                    "memories": [
                        {
                            "event": m.event,
                            "timestamp": m.timestamp,
                            "type": m.type.value,
                            "importance": m.importance,
                            "age": m.age,
                        }
                        for m in npc.memories
                    ],
                    "long_term_memories": npc.long_term_memories,
                }
                npc_data.append(npc_info)

            with open(os.path.join(world_dir, "npcs.json"), "w") as f:
                json.dump(npc_data, f)

            logger.debug(f"Saved world with seed {world.seed}")
        except Exception as e:
            logger.error(f"Failed to save world: {e}")

    def load_world(self, seed):
        """Load world state including NPCs and their memories"""
        world_dir = self.get_world_path(seed)
        if not os.path.exists(world_dir):
            logger.error(f"No save found for seed {seed}")
            return None

        try:
            # Load world data
            with open(os.path.join(world_dir, "world.json"), "r") as f:
                world_data = json.load(f)

            # Load NPCs data
            with open(os.path.join(world_dir, "npcs.json"), "r") as f:
                npc_data = json.load(f)

            return world_data, npc_data

        except Exception as e:
            logger.error(f"Error loading save for seed {seed}: {e}")
            return None

    def get_available_saves(self):
        """Get list of available saves"""
        saves = []
        for item in os.listdir(self.save_dir):
            if item.startswith("world_"):
                try:
                    world_path = os.path.join(self.save_dir, item, "world.json")
                    if os.path.exists(world_path):
                        with open(world_path, "r") as f:
                            data = json.load(f)
                            saves.append(
                                {
                                    "seed": data["seed"],
                                    "timestamp": data["timestamp"],
                                    "path": item,
                                }
                            )
                except Exception as e:
                    logger.error(f"Error reading save {item}: {e}")
        return saves
