import random
import noise
import math
from entities.world_object import WorldObject
from entities.npc import NPC
import logging

from utils.name_generator import NameGenerator

logger = logging.getLogger(__name__)


class World:
    def __init__(self, seed=None):
        logger.info(f"Initializing world with seed {seed}")
        self.size = 3000
        self.objects = []
        self.npcs = []
        self.seed = seed if seed is not None else random.randint(1, 99999)
        self.village_centers = []
        self.min_house_distance = 120
        self.current_weather = "sunny"  # Default weather
        random.seed(self.seed)
        self.generate_world()

    def check_collision(
        self, x: float, y: float, radius: float, ignore_entity=None
    ) -> bool:
        """Check if a position would result in a collision"""
        # Check world boundaries
        if (
            x - radius < 0
            or x + radius > self.size
            or y - radius < 0
            or y + radius > self.size
        ):
            return True

        # Check collision with solid objects
        for obj in self.objects:
            if not getattr(obj, "solid", False):  # Skip non-solid objects
                continue

            distance = math.sqrt((x - obj.x) ** 2 + (y - obj.y) ** 2)
            min_distance = radius + obj.radius

            if distance < min_distance:
                return True

        # Check collision with NPCs (if not checking for an NPC itself)
        if ignore_entity is None or not isinstance(ignore_entity, NPC):
            for npc in self.npcs:
                if npc == ignore_entity:
                    continue

                distance = math.sqrt((x - npc.x) ** 2 + (y - npc.y) ** 2)
                min_distance = radius + npc.radius

                if distance < min_distance:
                    return True

        return False

    def is_position_valid(
        self, x: float, y: float, radius: float, min_distance: float = None
    ) -> bool:
        """Check if a position is valid for object placement"""
        # Check world boundaries
        if (
            x - radius < 0
            or x + radius > self.size
            or y - radius < 0
            or y + radius > self.size
        ):
            return False

        # Check distance from other objects
        for obj in self.objects:
            if getattr(obj, "solid", False):  # Only check solid objects
                dist = math.sqrt((x - obj.x) ** 2 + (y - obj.y) ** 2)
                min_dist = min_distance if min_distance else (radius + obj.radius)
                if dist < min_dist:
                    return False
        return True

    def generate_village(self, center_x, center_y, size):
        """Generate a village with better spacing and organization"""
        houses = []

        # Create central plaza with well
        well = WorldObject(center_x, center_y, "well")
        well.render_layer = 2
        well.solid = True
        self.objects.append(well)

        # Generate houses in a circular pattern with better spacing
        num_houses = random.randint(4, 6)  # Fewer houses for better spacing
        base_distance = 150  # Increased base distance from center

        for i in range(num_houses):
            angle = (i / num_houses) * 2 * math.pi
            distance = random.randint(base_distance, base_distance + 60)
            attempts = 0

            while attempts < 10:
                house_x = center_x + math.cos(angle) * distance
                house_y = center_y + math.sin(angle) * distance

                # Add slight randomness to position
                house_x += random.randint(-20, 20)
                house_y += random.randint(-20, 20)

                if self.is_position_valid(
                    house_x, house_y, 30, self.min_house_distance
                ):
                    house = WorldObject(house_x, house_y, "house")
                    house.solid = True
                    house.render_layer = 2  # Top layer
                    houses.append(house)
                    self.objects.append(house)

                    # Create paths from house to well
                    path_tiles = self.create_path(
                        (house_x, house_y), (center_x, center_y)
                    )
                    self.objects.extend(path_tiles)

                    # Add decorative elements around house
                    self.add_house_decorations(house_x, house_y)

                    # Add NPC
                    self.add_house_npc(house_x, house_y)
                    break

                attempts += 1

        return houses

    def add_house_decorations(self, house_x, house_y):
        """Add decorative elements around a house"""
        for _ in range(3):
            attempts = 0
            while attempts < 5:
                dx = random.randint(-25, 25)
                dy = random.randint(-25, 25)
                dec_x = house_x + dx
                dec_y = house_y + dy

                if self.is_position_valid(dec_x, dec_y, 5):
                    if random.random() < 0.5:
                        self.objects.append(WorldObject(dec_x, dec_y, "flower"))
                    else:
                        self.objects.append(WorldObject(dec_x, dec_y, "berry_bush"))
                    break
                attempts += 1

    def add_house_npc(self, house_x, house_y):
        """Add an NPC near a house in a valid position"""
        # Always try to place NPC on the path in front of the house
        attempts = 0
        while attempts < 30:  # More attempts
            # Try to place NPC on the path side of the house
            angle = random.uniform(0, 2 * math.pi)
            distance = random.randint(40, 60)  # Much larger distance from house

            npc_x = house_x + math.cos(angle) * distance
            npc_y = house_y + math.sin(angle) * distance

            # More thorough position validation
            valid_position = (
                0 <= npc_x <= self.size
                and 0 <= npc_y <= self.size
                and
                # Check for path nearby (NPCs prefer to stand near paths)
                any(
                    math.sqrt((obj.x - npc_x) ** 2 + (obj.y - npc_y) ** 2) < 30
                    for obj in self.objects
                    if getattr(obj, "is_path", False)
                )
                and
                # Ensure no collision with solid objects
                not any(
                    math.sqrt((obj.x - npc_x) ** 2 + (obj.y - npc_y) ** 2) < 30
                    for obj in self.objects
                    if getattr(obj, "solid", False)
                )
            )

            if valid_position:
                npc = NPC(npc_x, npc_y, f"Villager_{len(self.npcs)}")
                npc.home_x = house_x
                npc.home_y = house_y
                self.npcs.append(npc)
                logger.info(f"Created NPC {npc.name} at ({npc_x:.2f}, {npc_y:.2f})")
                return True

            attempts += 1

        logger.warning(
            f"Failed to place NPC for house at ({house_x:.2f}, {house_y:.2f})"
        )
        return False

    def create_water_body(self, x, y, size):
        """Create a water body using tiles"""
        water_tiles = []
        tile_size = 20

        for i in range(-size, size + 1):
            for j in range(-size, size + 1):
                # Create circular-ish water body
                if (i * i + j * j) <= size * size:
                    water_x = x + i * tile_size
                    water_y = y + j * tile_size

                    if self.is_position_valid(water_x, water_y, tile_size / 2):
                        water = WorldObject(water_x, water_y, "water")
                        water.is_water = True
                        water_tiles.append(water)

        return water_tiles

    def create_river(self, start_x, start_y, length, width=30):
        """Generate a meandering river using rectangular tiles"""
        river_tiles = []
        current_x, current_y = start_x, start_y
        angle = random.uniform(0, 2 * math.pi)
        tile_size = 20  # Size of each water tile

        for _ in range(length):
            # Meander the river
            angle += random.uniform(-0.2, 0.2)

            # Create main river tiles (rectangular pattern)
            for w in range(-width // 2, width // 2, tile_size):
                perpendicular = angle + math.pi / 2
                offset_x = math.cos(perpendicular) * w
                offset_y = math.sin(perpendicular) * w

                water = WorldObject(current_x + offset_x, current_y + offset_y, "water")
                water.render_layer = 0
                water.is_water = True
                water.solid = True
                water.width = tile_size  # Add width for rectangular rendering
                water.height = tile_size  # Add height for rectangular rendering
                river_tiles.append(water)

            # Move to next position
            current_x += math.cos(angle) * (
                tile_size / 2
            )  # Overlap slightly for continuity
            current_y += math.sin(angle) * (tile_size / 2)

            # Add riverbank decoration
            if random.random() < 0.1:
                side = random.choice([-1, 1])
                bank_x = current_x + math.cos(angle + math.pi / 2) * width * side
                bank_y = current_y + math.sin(angle + math.pi / 2) * width * side

                decor = WorldObject(
                    bank_x, bank_y, random.choice(["tall_grass", "flower"])
                )
                decor.render_layer = 1
                river_tiles.append(decor)

        return river_tiles

    def create_forest(self, center_x, center_y, radius):
        """Generate a forest area"""
        forest_objects = []
        num_trees = int(radius * radius / 1000)  # Scale number of trees with area

        for _ in range(num_trees):
            angle = random.uniform(0, 2 * math.pi)
            distance = random.uniform(0, radius)
            tree_x = center_x + math.cos(angle) * distance
            tree_y = center_y + math.sin(angle) * distance

            if self.is_position_valid(tree_x, tree_y, 15):
                tree = WorldObject(tree_x, tree_y, "tree")
                tree.render_layer = 2
                tree.solid = True
                forest_objects.append(tree)

                # Add forest floor decoration
                if random.random() < 0.3:
                    decor_x = tree_x + random.uniform(-20, 20)
                    decor_y = tree_y + random.uniform(-20, 20)
                    decor = WorldObject(
                        decor_x,
                        decor_y,
                        random.choice(["mushroom", "tall_grass", "flower"]),
                    )
                    decor.render_layer = 1
                    forest_objects.append(decor)

        return forest_objects

    def generate_world(self):
        self.objects = []
        self.npcs = []
        self.village_centers = []

        # Generate rivers first
        num_rivers = random.randint(2, 4)
        for _ in range(num_rivers):
            start_x = random.randint(0, self.size)
            start_y = random.randint(0, self.size)
            river_tiles = self.create_river(
                start_x, start_y, length=random.randint(30, 50)
            )
            self.objects.extend(river_tiles)

        # Generate forests
        num_forests = random.randint(4, 8)
        for _ in range(num_forests):
            forest_x = random.randint(200, self.size - 200)
            forest_y = random.randint(200, self.size - 200)

            # Check if position is valid (not in water)
            if not any(
                obj.is_water
                for obj in self.objects
                if math.sqrt((obj.x - forest_x) ** 2 + (obj.y - forest_y) ** 2) < 100
            ):
                forest = self.create_forest(
                    forest_x, forest_y, random.randint(150, 300)
                )
                self.objects.extend(forest)

        # Generate villages with proper spacing
        num_villages = random.randint(3, 5)
        min_village_distance = 400

        for i in range(num_villages):
            attempts = 0
            while attempts < 50:
                center_x = random.randint(300, self.size - 300)
                center_y = random.randint(300, self.size - 300)

                # Check if position is valid (not in water or forest)
                if (
                    not any(
                        obj.is_water
                        for obj in self.objects
                        if math.sqrt((obj.x - center_x) ** 2 + (obj.y - center_y) ** 2)
                        < 200
                    )
                    and not any(
                        obj.type == "tree"
                        for obj in self.objects
                        if math.sqrt((obj.x - center_x) ** 2 + (obj.y - center_y) ** 2)
                        < 150
                    )
                    and (
                        not self.village_centers
                        or all(
                            math.sqrt((x - center_x) ** 2 + (y - center_y) ** 2)
                            > min_village_distance
                            for x, y in self.village_centers
                        )
                    )
                ):

                    self.village_centers.append((center_x, center_y))
                    break
                attempts += 1

        # Generate paths between villages (avoiding water)
        for i in range(len(self.village_centers)):
            for j in range(i + 1, len(self.village_centers)):
                path_tiles = self.create_path(
                    self.village_centers[i], self.village_centers[j]
                )
                self.objects.extend(path_tiles)

        # Generate villages
        for center_x, center_y in self.village_centers:
            self.generate_village(center_x, center_y, random.randint(80, 120))

        # Sort objects by render layer
        self.objects.sort(key=lambda obj: getattr(obj, "render_layer", 1))

    def create_path(self, start, end):
        """Create a path between two points using rectangular tiles"""
        path_tiles = []
        dx = end[0] - start[0]
        dy = end[1] - start[1]
        distance = math.sqrt(dx * dx + dy * dy)
        tile_size = 20
        steps = int(distance / (tile_size / 2))  # Overlap tiles slightly for continuity

        # Add some natural curve to the path
        curve_strength = random.uniform(0.1, 0.3)
        mid_point = (
            (start[0] + end[0]) / 2 + random.randint(-100, 100) * curve_strength,
            (start[1] + end[1]) / 2 + random.randint(-100, 100) * curve_strength,
        )

        # Create main path tiles
        for i in range(steps + 1):
            t = i / steps
            # Quadratic Bezier curve
            x = (1 - t) ** 2 * start[0] + 2 * (1 - t) * t * mid_point[0] + t**2 * end[0]
            y = (1 - t) ** 2 * start[1] + 2 * (1 - t) * t * mid_point[1] + t**2 * end[1]

            path = WorldObject(x, y, "path")
            path.render_layer = 0  # Bottom layer
            path.is_path = True
            path.solid = False  # Paths are walkable
            path_tiles.append(path)

            # Add path borders (decorative)
            if random.random() < 0.05:  # Reduced frequency of decorations
                offset = random.randint(-15, 15)
                if random.random() < 0.5:
                    flower = WorldObject(x + offset, y + offset, "flower")
                    flower.render_layer = 1
                    path_tiles.append(flower)

        return path_tiles

    def create_village_paths(self, houses, well):
        """Create paths connecting houses to the central well"""
        for house in houses:
            path_tiles = self.create_path((house.x, house.y), (well.x, well.y))
            self.objects.extend(path_tiles)

    def find_valid_spawn(self, x, y, radius=15, max_attempts=100):
        """Find a nearby valid spawn point that doesn't collide with solids"""
        original_x, original_y = x, y
        attempt = 0

        while attempt < max_attempts:
            # Try the current position
            if not self.check_collision(x, y, radius):
                return x, y

            # Spiral out from original position
            angle = 2 * math.pi * attempt / max_attempts
            distance = (attempt / max_attempts) * 200  # Max 200 pixels from original
            x = original_x + math.cos(angle) * distance
            y = original_y + math.sin(angle) * distance

            attempt += 1

        # If no valid position found, try a completely random position
        for _ in range(max_attempts):
            x = random.randint(radius, self.size - radius)
            y = random.randint(radius, self.size - radius)
            if not self.check_collision(x, y, radius):
                return x, y

        logger.warning("Could not find valid spawn point!")
        return original_x, original_y  # Return original as last resort

    def generate_npcs(self):
        """Generate NPCs in valid positions near village centers"""
        for village_x, village_y in self.village_centers:
            num_npcs = random.randint(3, 6)
            for _ in range(num_npcs):
                base_x = village_x + random.randint(-100, 100)
                base_y = village_y + random.randint(-100, 100)
                # Find valid spawn point near the intended position
                x, y = self.find_valid_spawn(base_x, base_y)
                npc = NPC(x, y, NameGenerator.generate_name())
                self.npcs.append(npc)
